/*-- $Id: XercesParser.java,v 1.11 2000/10/15 22:51:31 greenrd Exp $ --

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2001 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Cocoon" and  "Apache Software Foundation"  must not be used to
    endorse  or promote  products derived  from this  software without  prior
    written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

 */

package org.apache.cocoon.parser;

import java.io.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import org.apache.xerces.dom.*;
import org.apache.xerces.parsers.*;
import org.apache.xerces.framework.*;
import org.apache.cocoon.framework.*;

/**
 * This class implements an XML parser using the Apache Xerces XML parser.
 *
 * @author <a href="mailto:stefano@apache.org">Stefano Mazzocchi</a>
 * @version $Revision: 1.11 $ $Date: 2000/10/15 22:51:31 $
 */

public class XercesParser extends AbstractParser implements Status {

    /**
     * Creates a DOM tree parsing the given input source.
     */
    public Document parse(InputSource input, boolean validate) throws SAXException, IOException {
        DOMParser parser = new DOMParser();
        parser.setErrorHandler(this);
        parser.setFeature("http://xml.org/sax/features/validation", validate);
        parser.setFeature("http://apache.org/xml/features/validation/schema", false);
        parser.setFeature("http://xml.org/sax/features/namespaces", true);        
        parser.setFeature("http://apache.org/xml/features/validation/warn-on-duplicate-attdef", validate);
        parser.setFeature("http://apache.org/xml/features/validation/warn-on-undeclared-elemdef", validate);
        parser.setFeature("http://apache.org/xml/features/dom/create-entity-ref-nodes", false);
        parser.setFeature("http://apache.org/xml/features/dom/defer-node-expansion", true);
        parser.setFeature("http://apache.org/xml/features/allow-java-encodings", true);
        parser.parse(resolveSystemId(input));
        return parser.getDocument();
    }

    /*
     * Make sure the SystemID is fully resolved as the SAX2 spec recommends.
     */
    private InputSource resolveSystemId(InputSource input) {
        String id = input.getSystemId();
        if (id != null) {
            id = id.replace('\\','/');
            if ((id.indexOf("://") < 0) && (!id.startsWith("file:"))) {
                id = "file:" + id;
            }
            input.setSystemId(id);
        }
        return input;
    }
    
    /**
     * Creates an empty DOM tree.
     */
    public Document createEmptyDocument() {
        return new DocumentImpl();
    }

    public String getStatus() {
        return Version.fVersion + " XML Parser (validation = " + validation + ")";
    }
}
